package sun.java2d.d3d;

import sun.java2d.pipe.BufferedContext;
import sun.java2d.pipe.hw.AccelDeviceEventNotifier;
import sun.java2d.pipe.hw.AccelDeviceEventListener;
import sun.java2d.pipe.hw.ContextCapabilities;
import sun.java2d.Surface;
import sun.java2d.pipe.hw.AccelSurface;
import sun.java2d.pipe.hw.AccelTypedVolatileImage;
import java.awt.Graphics;
import sun.awt.image.SurfaceManager;
import sun.awt.image.SunVolatileImage;
import java.awt.image.VolatileImage;
import java.awt.AWTException;
import java.awt.Component;
import sun.awt.windows.WComponentPeer;
import java.awt.color.ColorSpace;
import java.awt.image.DirectColorModel;
import java.awt.image.ColorModel;
import sun.java2d.SurfaceData;
import java.awt.BufferCapabilities;
import java.awt.ImageCapabilities;
import sun.java2d.pipe.hw.AccelGraphicsConfig;
import sun.awt.Win32GraphicsConfig;

public class D3DGraphicsConfig extends Win32GraphicsConfig implements AccelGraphicsConfig
{
    private static ImageCapabilities imageCaps;
    private BufferCapabilities bufferCaps;
    private D3DGraphicsDevice device;
    
    protected D3DGraphicsConfig(D3DGraphicsDevice device) {
        super(device, 0);
        this.device = device;
    }
    
    public SurfaceData createManagedSurface(int n, int n2, int n3) {
        return D3DSurfaceData.createData(this, n, n2, this.getColorModel(n3), null, 3);
    }
    
    public synchronized void displayChanged() {
        super.displayChanged();
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            D3DContext.invalidateCurrentContext();
        }
        finally {
            instance.unlock();
        }
    }
    
    public ColorModel getColorModel(int n) {
        switch (n) {
            case 1: {
                return new DirectColorModel(24, 16711680, 65280, 255);
            }
            case 2: {
                return new DirectColorModel(25, 16711680, 65280, 255, 16777216);
            }
            case 3: {
                return new DirectColorModel(ColorSpace.getInstance(1000), 32, 16711680, 65280, 255, -16777216, true, 3);
            }
            default: {
                return null;
            }
        }
    }
    
    public String toString() {
        return "D3DGraphicsConfig[dev=" + this.screen + ",pixfmt=" + this.visual + "]";
    }
    
    public SurfaceData createSurfaceData(WComponentPeer wComponentPeer, int n) {
        return super.createSurfaceData(wComponentPeer, n);
    }
    
    public void assertOperationSupported(Component component, int n, BufferCapabilities bufferCapabilities) throws AWTException {
        if (n < 2 || n > 4) {
            throw new AWTException("Only 2-4 buffers supported");
        }
        if (bufferCapabilities.getFlipContents() == BufferCapabilities.FlipContents.COPIED && n != 2) {
            throw new AWTException("FlipContents.COPIED is onlysupported for 2 buffers");
        }
        return;
    }
    
    public VolatileImage createBackBuffer(WComponentPeer wComponentPeer) {
        Component comp = (Component)wComponentPeer.getTarget();
        return new SunVolatileImage(comp, Math.max(1, comp.getWidth()), Math.max(1, comp.getHeight()), Boolean.TRUE);
    }
    
    public void flip(WComponentPeer wComponentPeer, Component component, VolatileImage img, int n, int n2, int n3, int n4, BufferCapabilities.FlipContents flipContents) {
        SurfaceData primarySurfaceData = SurfaceManager.getManager(img).getPrimarySurfaceData();
        if ((primarySurfaceData instanceof D3DSurfaceData)) {
            D3DSurfaceData.swapBuffers((D3DSurfaceData)primarySurfaceData, n, n2, n3, n4);
        }
        else {
            Graphics graphics = wComponentPeer.getGraphics();
            try {
                graphics.drawImage(img, n, n2, n3, n4, n, n2, n3, n4, null);
            }
            finally {
                graphics.dispose();
            }
        }
        if (flipContents == BufferCapabilities.FlipContents.BACKGROUND) {
            Graphics graphics2 = img.getGraphics();
            try {
                graphics2.setColor(component.getBackground());
                graphics2.fillRect(0, 0, img.getWidth(), img.getHeight());
            }
            finally {
                graphics2.dispose();
            }
        }
    }
    
    public BufferCapabilities getBufferCapabilities() {
        if (this.bufferCaps == null) {
            this.bufferCaps = new D3DGraphicsConfig.D3DBufferCaps();
        }
        return this.bufferCaps;
    }
    
    public ImageCapabilities getImageCapabilities() {
        return D3DGraphicsConfig.imageCaps;
    }
    
    D3DGraphicsDevice getD3DDevice() {
        return this.device;
    }
    
    public D3DContext getContext() {
        return this.device.getContext();
    }
    
    public VolatileImage createCompatibleVolatileImage(int width, int height, int transparency, int accType) {
        if (accType == 4 || accType == 1 || accType == 0 || transparency == 2) {
            return null;
        }
        int n = transparency == 1;
        if (accType == 5) {
            if (!this.device.isCapPresent(n ? 8 : 4)) {
                return null;
            }
        }
        else if (accType == 2 && n == 0 && !this.device.isCapPresent(2)) {
            return null;
        }
        AccelTypedVolatileImage o = new AccelTypedVolatileImage(this, width, height, transparency, accType);
        Surface destSurface = o.getDestSurface();
        if (!(destSurface instanceof AccelSurface) || ((AccelSurface)destSurface).getType() != accType) {
            o.flush();
            o = null;
            return o;
        }
        return o;
    }
    
    public ContextCapabilities getContextCapabilities() {
        return this.device.getContextCapabilities();
    }
    
    public void addDeviceEventListener(AccelDeviceEventListener accelDeviceEventListener) {
        AccelDeviceEventNotifier.addListener(accelDeviceEventListener, this.device.getScreen());
    }
    
    public void removeDeviceEventListener(AccelDeviceEventListener accelDeviceEventListener) {
        AccelDeviceEventNotifier.removeListener(accelDeviceEventListener);
    }
    
    static {
        D3DGraphicsConfig.imageCaps = new D3DGraphicsConfig.D3DImageCaps();
    }
    
    private static class D3DBufferCaps extends BufferCapabilities
    {
        public D3DBufferCaps() {
            super(D3DGraphicsConfig.imageCaps, D3DGraphicsConfig.imageCaps, BufferCapabilities.FlipContents.UNDEFINED);
        }
        
        @Override
        public boolean isMultiBufferAvailable() {
            return true;
        }
    }
    
    private static class D3DImageCaps extends ImageCapabilities
    {
        private D3DImageCaps() {
            super(true);
        }
        
        @Override
        public boolean isTrueVolatile() {
            return true;
        }
    }
}
